//---------------------------------------------------------------------------
//  watek odczytu danych z USB

#include "ThreadComm.h"
#include "usb.h"
#include <stdio.h>
#include "..\miniscope\device\MiniscopeDevice.h"
#include "ScopedLock.h"
#include "armscope_cmd.h"

#pragma package(smart_init)

bool writeLock;

__fastcall ThreadComm::ThreadComm(bool CreateSuspended)
		: callbackLog(NULL), callbackDataRcv(NULL),
		connected(true),
		rxFifo(64, E_USB_FRAME_SIZE)
{
}
//---------------------------------------------------------------------------


void ThreadComm::log(char *lpData, ...)
{
    static char buf[256]; //determines max message length
	va_list ap;
	va_start(ap, lpData);
	vsnprintf(buf, sizeof(buf), lpData, ap);
	va_end(ap);
	buf[sizeof(buf)-1] = 0;

	if (callbackLog)
		callbackLog(buf);
}

//---------------------------------------------------------------------------
DWORD WINAPI ThreadCommProc(LPVOID data)
{
    ThreadComm *thrRead = (ThreadComm*)data;

	while (thrRead->connected)
	{
		static unsigned char usb_rcv[ThreadComm::E_USB_FRAME_SIZE];
		int ret = usb_bulk_read( usb_handle, 0x82, (char*)usb_rcv, ThreadComm::E_USB_FRAME_SIZE, 25 );
		if (ret < 0) {
			if (ret != -116)	//timeout
			{
				thrRead->log("usb_bulk_read: status = %d, %s", ret, usb_strerror());
				thrRead->connected = false;
				if (thrRead->callbackConnect)
					thrRead->callbackConnect(DEVICE_DISCONNECTED, "usb_bulk_read failed, disconnected");
			}
		}

		if (ret > 0)
		{
			int status = thrRead->rxFifo.put(usb_rcv, ret);
			if (status)
            	thrRead->log("ThreadComm: RX FIFO overflowed!");
		}

		bool bSendRequest = false;
		ThreadComm::Cmd scmd;
		{
			ScopedLock<Mutex> lock(thrRead->mutexSend);
			if (!thrRead->que_cmd.empty())
			{
				scmd = thrRead->que_cmd.front();
				thrRead->que_cmd.pop_front();
				bSendRequest = true;
			}
		}
		if (bSendRequest)
		{
			char buffer[3];
			buffer[0] = scmd.cmd;
			buffer[1] = scmd.val;
			buffer[2] = 0x00;
			int ret = usb_bulk_write(usb_handle, 0x01, buffer, 3, 500);
			if (ret < 3)
				thrRead->log("usb_bulk_write: status = %d, %s", ret, usb_strerror());
		}
	}
	return 0;
}
//---------------------------------------------------------------------------

DWORD WINAPI ThreadRxQueueProc(LPVOID data)
{
    ThreadComm *thrRead = (ThreadComm*)data;

	while (thrRead->connected)
	{
		unsigned char* usb_rcv;
		int len = thrRead->rxFifo.get(&usb_rcv);
		if (len > 0)
		{
			if (thrRead->callbackDataRcv) {
				thrRead->callbackDataRcv(usb_rcv, len);
			}
		}
		Sleep(5);
	}
	return 0;
}

void ThreadComm::Execute(void)
{
	DWORD dwtid;
	Thread = CreateThread(NULL, 0, ThreadCommProc, this, 0, &dwtid);
	if (Thread)
	{
		bool status = SetThreadPriority(Thread, THREAD_PRIORITY_TIME_CRITICAL);
		if (!status)
			log("Failed to set communication thread priority!");
	}
	else
	{
		log("Failed to create communication thread!");
	}

	ThreadRxQueue = CreateThread(NULL, 0, ThreadRxQueueProc, this, 0, &dwtid);
	if (ThreadRxQueue)
	{
		bool status = SetThreadPriority(ThreadRxQueue, THREAD_PRIORITY_ABOVE_NORMAL);
		if (!status)
			log("Failed to set data queue thread priority!");
	}
	else
	{
		log("Failed to create data queue thread!");
	}
}

int ThreadComm::SendCmd(unsigned char cmd, unsigned char val)
{
	if (!connected)
	{
		return -1;
	}
	else
	{
		//ScopedLock<Mutex> lock(mutexSend);
		Cmd scmd;
		scmd.cmd = cmd;
		scmd.val = val;
		mutexSend.lock();
		que_cmd.push_back(scmd);
		mutexSend.unlock();
		return 0;
	}
}


